function residual = residual_analysis(data,res,FS,make_plot);
% This function performs a residual ananlysis of MoCap data (or similiar
% digital signals). The residual can be used to determine the frequency
% content of a digital signal and a reasonable cut-off frequency. Due to
% the filtering process the input data should have a lenght of more than
% twice the used filter lenght <res>. (See comments about the filter order
% below.)
% 
% See [1] and [2] for more information about this method and the
% interpretation of the residual result.
%
% Usage: [residual] = residual_analysis(data,FS,make_plot);
%   data    - The analysed MoCap data or similar data. Can be a vector,
%             e.g. three dimensional positional data.
%   res     - Determines the order of the used filter in the residual
%             analysis. An order of 200 works good for most MoCap data.
%             Higher orders will give sharper filters and therefore sharper
%             residual plots. However, before we can calcualte the residual
%             we need to trim the filtered data with halv of the filter
%             lenght due to filters edge effect. This limits the used
%             filter order if the method is used on data with limited
%             length.
%   FS      - Sampling frequency (only used for the plot)
%   pl      - If <1> the function plots the resulting residual, <0> no plot
%
% See the <residual_analysis_example.m> file for how to use this method.
% 
% [1] Skogstad, S. et al., Filtering Motion Capture Data for Real-Time
% Applications, In proceedings of NIME (2013)
% [2] Winter, D.A., Biomechanics and Motor Control of Human Movement, John
% Wiley & Sons (2009) 
%
% Written by Stle A. Skogstad (02.04.2013), fourMs lab, University of Oslo
% http://fourms.uio.no

% We want to filter with different cut-off freqeuncy (normalized
% frequency). Here can a higher resolution be used if wanted. 
pbe = 0.01:0.01:0.98;

% The residual will be saved in the following vector.
residual = zeros(length(pbe),1);

idx = 0; % reset the index.

% Now we are ready to filter the data with different cut-off freqyencies.
for pb = pbe
    idx = idx + 1; % Index of the current cut-off frequency
    
    % Make FIR filter <b> with cut-off frequency <pb> and filter the data.
    % Here is the window method used (fir1). It is possible to use other
    % filter design methods, but this method was found to be the most
    % versatile, especially since it works for a wide cut-off frequency
    % range.
    b    = fir1(res,pb);
    tmpf = filter(b,1, data);
    
    % Before we can compare filtered and unfiltered data we need to adjust
    % for the time shift and trim the data due to the filter's edge effect.
    filMD = tmpf(res/2+1:end,:); 
    len   = length(filMD);
    orgMD = data(1:len,:);
    filMD = filMD(res/2:end,:);      
    orgMD = orgMD(res/2:end,:);

    % Compare filtered with unfiltered data (compute difference)
    diff = orgMD-filMD; 

    % Compute RMS difference, i.e. the residual.
    diffV = sqrt(sum(diff.^2,2));             % Compute distance of difference
    residual(idx) = sum(diffV)/length(diffV); % Compute the RMS of, i.e. the residual
end

% If pl is not 0 the method plot the residual curve
if make_plot
    figure
    plot(pbe(1:end)*FS/2, residual(1:end));
    grid on;
    title('Residual - difference between raw and filtered data')
    xlabel('cutoff frequency (Hz)');
    ylabel('residual RMS'); % annotation depends on how the data was originally given
end

